"{ Package: 'stx:goodies/xmlsuite/xquery' }"

"{ NameSpace: XQuery }"

TestCase subclass:#XQueryInterpreterTests
	instanceVariableNames:'interpreter'
	classVariableNames:''
	poolDictionaries:''
	category:'XQuery-Tests'
!


!XQueryInterpreterTests class methodsFor:'accessing'!

resources

    ^Array 
        with:XQueryDocumentProviderTestResource

    "Created: / 06-04-2007 / 17:18:09 / janfrog"
! !

!XQueryInterpreterTests class methodsFor:'defaults'!

testedURI
    ^'file:///bookstore.xml'.

    "Created: / 02-12-2006 / 12:11:18 / ked"
    "Modified: / 24-12-2006 / 17:41:19 / janfrog"
! !

!XQueryInterpreterTests methodsFor:'initialization & release'!

setUp

    interpreter := XQueryInterpreter new
                        setDocumentProvider: XQueryDocumentProviderTestResource current;
                        yourself

    "Created: / 25-10-2006 / 09:08:14 / ked"
    "Modified: / 14-11-2007 / 12:23:58 / janfrog"
    "Modified: / 18-09-2008 / 16:45:34 / Jan Vrany <vranyj1@fel.cvut.cz>"
!

tearDown

    interpreter := nil.

    "Created: / 25-10-2006 / 09:08:15 / ked"
    "Modified: / 18-09-2008 / 16:45:20 / Jan Vrany <vranyj1@fel.cvut.cz>"
! !

!XQueryInterpreterTests methodsFor:'tests'!

testAdditiveExpr
    |result|

    result := interpreter basicEvaluate:'1 + 2.0'.
    self assert:result size = 1.
    self assert:result first type = XQuerySequenceItem typeXSFloat.
    self assert:result first value = 3.0.

    "Modified: / 03-03-2007 / 11:04:23 / ked"
    "Created: / 29-03-2007 / 09:58:14 / janfrog"
    "Modified: / 05-12-2007 / 15:15:54 / janfrog"
    "Modified: / 18-09-2008 / 16:57:36 / Jan Vrany <vranyj1@fel.cvut.cz>"
!

testConstructor2
    |result resultChild resultAttribute|

    result := interpreter 
                basicEvaluate:'document { element a { attribute atr { "atr" }, element b {}, text { "element a" } } }'.
    self assert:result size = 1.
    self assert:result first item xpathIsDocument.
    result := result first item xpathChild.
    self assert:result size = 1.
    self assert:result first xpathIsElement.
    self assert:result first xpathName = 'a'.
    resultChild := result first xpathChild.
    self assert:resultChild size = 2.
    self assert:resultChild first xpathIsElement.
    self assert:resultChild first xpathName = 'b'.
    self assert:resultChild second xpathIsText.
    self assert:resultChild second xpathValue = 'element a'.
    resultAttribute := result first xpathAttribute.
    self assert:resultAttribute size = 1.
    self assert:resultAttribute first xpathIsAttribute.
    self assert:resultAttribute first xpathName = 'atr'.
    self assert:resultAttribute first xpathValue = 'atr'.

    "Created: / 22-11-2006 / 16:52:42 / ked"
    "Modified: / 03-03-2007 / 11:04:44 / ked"
    "Modified: / 18-11-2007 / 09:42:58 / janfrog"
    "Modified: / 18-09-2008 / 16:57:38 / Jan Vrany <vranyj1@fel.cvut.cz>"
!

testExpr
    |result|

    result := interpreter basicEvaluate:'1, 2, 3, 5 to 7'.
    self assert:result size = 6.
    self 
        assert:(result first type isSubtypeOf:XQuerySequenceItem typeXSInteger).
    self assert:result first value = 1.
    self 
        assert:(result second type isSubtypeOf:XQuerySequenceItem typeXSInteger).
    self assert:result second value = 2.
    self 
        assert:(result third type isSubtypeOf:XQuerySequenceItem typeXSInteger).
    self assert:result third value = 3.
    self 
        assert:(result fourth type isSubtypeOf:XQuerySequenceItem typeXSInteger).
    self assert:result fourth value = 5.
    self 
        assert:(result fifth type isSubtypeOf:XQuerySequenceItem typeXSInteger).
    self assert:result fifth value = 6.
    self 
        assert:(result sixth type isSubtypeOf:XQuerySequenceItem typeXSInteger).
    self assert:result sixth value = 7.

    "Created: / 13-09-2006 / 16:28:01 / ked"
    "Modified: / 03-12-2006 / 13:36:01 / ked"
    "Modified: / 05-12-2007 / 15:29:47 / janfrog"
    "Modified: / 18-09-2008 / 16:57:36 / Jan Vrany <vranyj1@fel.cvut.cz>"
!

testFunctionCall1
    |result|

    result := interpreter basicEvaluate:'fn:sum((3, 4, 5))'.
    self assert:result size = 1.
    self 
        assert:(result first type isSubtypeOf:XQuerySequenceItem typeXSInteger).
    self assert:result first value = 12.

    "Created: / 29-03-2007 / 09:58:14 / janfrog"
    "Modified: / 05-12-2007 / 15:37:05 / janfrog"
    "Modified: / 18-09-2008 / 16:57:37 / Jan Vrany <vranyj1@fel.cvut.cz>"
!

testFunctionCall2
    |result|

    result := interpreter basicEvaluate:'fn:boolean(())'.
    self assert:result size = 1.
    self assert:result first type = XQuerySequenceItem typeBoolean.
    self assert:result first value = false.
    result := interpreter basicEvaluate:'fn:boolean(3)'.
    self assert:result size = 1.
    self assert:result first type = XQuerySequenceItem typeBoolean.
    self assert:result first value = true.

    "Created: / 29-03-2007 / 09:58:14 / janfrog"
    "Modified: / 18-09-2008 / 16:57:37 / Jan Vrany <vranyj1@fel.cvut.cz>"
!

testIfExpr
    |result|

    result := interpreter 
                basicEvaluate:'if (1.1 eq 1 and 1 eq 1)
     then "TRUE"
     else if ("a" ne "a")
          then 1
          else 2'.
    self assert:result size = 1.
    self 
        assert:(result first type isSubtypeOf:XQuerySequenceItem typeXSInteger).
    self assert:result first value = 2.

    "Created: / 13-09-2006 / 16:00:58 / ked"
    "Modified: / 03-12-2006 / 13:39:13 / ked"
    "Modified: / 05-12-2007 / 15:37:06 / janfrog"
    "Modified: / 18-09-2008 / 16:57:38 / Jan Vrany <vranyj1@fel.cvut.cz>"
!

testMultiplicativeExpr
    |result|

    result := interpreter basicEvaluate:'9 idiv 4'.
    self assert:result size = 1.
    self 
        assert:(result first type isSubtypeOf:XQuerySequenceItem typeXSInteger).
    self assert:result first value = 2.

    "Created: / 13-09-2006 / 16:21:36 / ked"
    "Modified: / 03-12-2006 / 13:39:26 / ked"
    "Modified: / 05-12-2007 / 15:37:06 / janfrog"
    "Modified: / 18-09-2008 / 16:57:36 / Jan Vrany <vranyj1@fel.cvut.cz>"
!

testParenthesizedExpr
    |result|

    result := interpreter basicEvaluate:'5 * (2 + 4)'.
    self assert:result size = 1.
    self 
        assert:(result first type isSubtypeOf:XQuerySequenceItem typeXSInteger).
    self assert:result first value = 30.
    result := interpreter basicEvaluate:'5 * 2 + 4'.
    self assert:result size = 1.
    self 
        assert:(result first type isSubtypeOf:XQuerySequenceItem typeXSInteger).
    self assert:result first value = 14.

    "Created: / 13-09-2006 / 16:24:25 / ked"
    "Modified: / 03-12-2006 / 13:39:45 / ked"
    "Modified: / 05-12-2007 / 15:36:02 / janfrog"
    "Modified: / 18-09-2008 / 16:57:37 / Jan Vrany <vranyj1@fel.cvut.cz>"
!

testQuantifiedExpr
    |result|

    result := interpreter 
                basicEvaluate:'if (every $x in (1, 2, 3), $y in (2, 3, 4) satisfies $x + $y eq 4)
     then "TRUE"
     else "FALSE"'.
    self assert:result size = 1.
    self 
        assert:(result first type isSubtypeOf:XQuerySequenceItem typeXSString).
    self assert:result first value = 'FALSE'.
    result := interpreter 
                basicEvaluate:'if (some $x in (1, 2, 3), $y in (2, 3, 4) satisfies $x + $y eq 4)
     then "TRUE"
     else "FALSE"'.
    self assert:result size = 1.
    self 
        assert:(result first type isSubtypeOf:XQuerySequenceItem typeXSString).
    self assert:result first value = 'TRUE'.

    "Created: / 13-09-2006 / 17:02:10 / ked"
    "Modified: / 03-12-2006 / 13:42:32 / ked"
    "Modified: / 05-12-2007 / 15:38:52 / janfrog"
    "Modified: / 18-09-2008 / 16:57:38 / Jan Vrany <vranyj1@fel.cvut.cz>"
!

testRangeExpr
    |result i|

    result := interpreter basicEvaluate:'1 to 10'.
    self assert:result size = 10.
    i := 1.
    result do:[:item | 
        self assert:(item type isSubtypeOf:XQuerySequenceItem typeXSInteger).
        self assert:item value = i.
        i := i + 1.
    ].

    "Created: / 13-09-2006 / 16:07:03 / ked"
    "Modified: / 03-12-2006 / 13:42:49 / ked"
    "Modified: / 05-12-2007 / 15:37:05 / janfrog"
    "Modified: / 18-09-2008 / 16:57:35 / Jan Vrany <vranyj1@fel.cvut.cz>"
! !

!XQueryInterpreterTests methodsFor:'tests - FLWOR'!

testFLWORExpr1
    |result|

    result := interpreter basicEvaluate:'let $i := (1, 2, 3) return $i'.
    self assert:result size = 3.
    self 
        assert:(result first type isSubtypeOf:XQuerySequenceItem typeXSInteger).
    self assert:result first value = 1.
    self 
        assert:(result second type isSubtypeOf:XQuerySequenceItem typeXSInteger).
    self assert:result second value = 2.
    self 
        assert:(result third type isSubtypeOf:XQuerySequenceItem typeXSInteger).
    self assert:result third value = 3.

    "Created: / 13-09-2006 / 16:29:35 / ked"
    "Modified: / 03-12-2006 / 13:36:19 / ked"
    "Modified: / 05-12-2007 / 15:30:14 / janfrog"
    "Modified: / 18-09-2008 / 16:57:37 / Jan Vrany <vranyj1@fel.cvut.cz>"
!

testFLWORExpr2
    |result|

    result := interpreter basicEvaluate:'for $j in (4, 5, 6) return $j'.
    self assert:result size = 3.
    self 
        assert:(result first type isSubtypeOf:XQuerySequenceItem typeXSInteger).
    self assert:result first value = 4.
    self 
        assert:(result second type isSubtypeOf:XQuerySequenceItem typeXSInteger).
    self assert:result second value = 5.
    self 
        assert:(result third type isSubtypeOf:XQuerySequenceItem typeXSInteger).
    self assert:result third value = 6.

    "Created: / 13-09-2006 / 16:42:20 / ked"
    "Modified: / 03-12-2006 / 13:36:40 / ked"
    "Modified: / 05-12-2007 / 15:30:37 / janfrog"
    "Modified: / 18-09-2008 / 16:57:35 / Jan Vrany <vranyj1@fel.cvut.cz>"
!

testFLWORExpr3
    |result|

    result := interpreter 
                basicEvaluate:'for $i in (1, 2), $j in (3, 4) return ($i, $j)'.
    self assert:result size = 8.
    self 
        assert:(result first type isSubtypeOf:XQuerySequenceItem typeXSInteger).
    self assert:result first value = 1.
    self 
        assert:(result second type isSubtypeOf:XQuerySequenceItem typeXSInteger).
    self assert:result second value = 3.
    self 
        assert:(result third type isSubtypeOf:XQuerySequenceItem typeXSInteger).
    self assert:result third value = 1.
    self 
        assert:(result fourth type isSubtypeOf:XQuerySequenceItem typeXSInteger).
    self assert:result fourth value = 4.
    self 
        assert:(result fifth type isSubtypeOf:XQuerySequenceItem typeXSInteger).
    self assert:result fifth value = 2.
    self 
        assert:(result sixth type isSubtypeOf:XQuerySequenceItem typeXSInteger).
    self assert:result sixth value = 3.
    self 
        assert:(result seventh type isSubtypeOf:XQuerySequenceItem typeXSInteger).
    self assert:result seventh value = 2.
    self 
        assert:((result at:8) type isSubtypeOf:XQuerySequenceItem typeXSInteger).
    self assert:(result at:8) value = 4.

    "Created: / 13-09-2006 / 16:46:47 / ked"
    "Modified: / 03-12-2006 / 13:37:25 / ked"
    "Modified: / 05-12-2007 / 15:37:05 / janfrog"
    "Modified: / 18-09-2008 / 16:57:36 / Jan Vrany <vranyj1@fel.cvut.cz>"
!

testFLWORExpr4
    |result|

    result := interpreter 
                basicEvaluate:'let $i := 5,
         $j := 20 * $i
     return ($i, $j)'.
    self assert:result size = 2.
    self 
        assert:(result first type isSubtypeOf:XQuerySequenceItem typeXSInteger).
    self assert:result first value = 5.
    self 
        assert:(result second type isSubtypeOf:XQuerySequenceItem typeXSInteger).
    self assert:result second value = 100.

    "Created: / 13-09-2006 / 16:29:35 / ked"
    "Modified: / 03-12-2006 / 13:37:42 / ked"
    "Modified: / 05-12-2007 / 15:35:57 / janfrog"
    "Modified: / 18-09-2008 / 16:57:37 / Jan Vrany <vranyj1@fel.cvut.cz>"
!

testFLWORExpr5
    |result|

    result := interpreter basicEvaluate:'for $a in (1 to 100) return $a'.
    self assert:result size = 100.
    1 to:100 do:[:i | 
        self 
            assert:((result at:i) type isSubtypeOf:XQuerySequenceItem typeXSInteger).
        self assert:(result at:i) value = i.
    ].

    "Created: / 13-09-2006 / 16:54:50 / ked"
    "Modified: / 03-12-2006 / 13:43:25 / ked"
    "Modified: / 05-12-2007 / 15:35:57 / janfrog"
    "Modified: / 18-09-2008 / 16:57:38 / Jan Vrany <vranyj1@fel.cvut.cz>"
!

testFLWORExpr6
    |result|

    result := interpreter 
                basicEvaluate:'for $a at $b in (5, 10, 15) return ($b, $a)'.
    self assert:result size = 6.
    self 
        assert:(result first type isSubtypeOf:XQuerySequenceItem typeXSInteger).
    self assert:result first value = 1.
    self 
        assert:(result second type isSubtypeOf:XQuerySequenceItem typeXSInteger).
    self assert:result second value = 5.
    self 
        assert:(result third type isSubtypeOf:XQuerySequenceItem typeXSInteger).
    self assert:result third value = 2.
    self 
        assert:(result fourth type isSubtypeOf:XQuerySequenceItem typeXSInteger).
    self assert:result fourth value = 10.
    self 
        assert:(result fifth type isSubtypeOf:XQuerySequenceItem typeXSInteger).
    self assert:result fifth value = 3.
    self 
        assert:(result sixth type isSubtypeOf:XQuerySequenceItem typeXSInteger).
    self assert:result sixth value = 15.

    "Created: / 13-09-2006 / 16:57:10 / ked"
    "Modified: / 03-12-2006 / 13:43:34 / ked"
    "Modified: / 05-12-2007 / 15:36:04 / janfrog"
    "Modified: / 18-09-2008 / 16:57:35 / Jan Vrany <vranyj1@fel.cvut.cz>"
!

testFLWORExpr7
    |result|

    result := interpreter 
                basicEvaluate:'for $a in (10, 15, 20, 25, 30)
     let $b := $a * 10
     where $b lt 230
     return
       if ($b mod 100 = 0)
       then ($b, "hundred :-)")
       else ($b, "no hundred :-(")'.
    self assert:result size = 6.
    self 
        assert:(result first type isSubtypeOf:XQuerySequenceItem typeXSInteger).
    self assert:result first value = 100.
    self 
        assert:(result second type isSubtypeOf:XQuerySequenceItem typeXSString).
    self assert:result second value = 'hundred :-)'.
    self 
        assert:(result third type isSubtypeOf:XQuerySequenceItem typeXSInteger).
    self assert:result third value = 150.
    self 
        assert:(result fourth type isSubtypeOf:XQuerySequenceItem typeXSString).
    self assert:result fourth value = 'no hundred :-('.
    self 
        assert:(result fifth type isSubtypeOf:XQuerySequenceItem typeXSInteger).
    self assert:result fifth value = 200.
    self 
        assert:(result sixth type isSubtypeOf:XQuerySequenceItem typeXSString).
    self assert:result sixth value = 'hundred :-)'.

    "Created: / 13-09-2006 / 17:00:01 / ked"
    "Modified: / 03-12-2006 / 13:39:00 / ked"
    "Modified: / 05-12-2007 / 15:38:52 / janfrog"
    "Modified: / 18-09-2008 / 16:57:36 / Jan Vrany <vranyj1@fel.cvut.cz>"
! !

!XQueryInterpreterTests methodsFor:'tests - arithmetic'!

test_arithmetic_additive_01
    |result|

    result := interpreter basicEvaluate:'1 + 2.0'.
    self assert:result size = 1.
    self 
        assert:(result first type isSubtypeOf:XQuerySequenceItem typeXSFloat).
    self assert:result first value = 3.0.

    "Modified: / 03-12-2006 / 13:30:25 / ked"
    "Created: / 21-03-2007 / 14:02:10 / janfrog"
    "Modified: / 05-12-2007 / 15:45:46 / janfrog"
    "Modified: / 18-09-2008 / 16:57:37 / Jan Vrany <vranyj1@fel.cvut.cz>"
!

test_arithmetic_additive_02
    |result|

    result := interpreter basicEvaluate:'1 - 1'.
    self assert:result size = 1.
    self 
        assert:(result first type isSubtypeOf:XQuerySequenceItem typeXSInteger).
    self assert:result first value = 0.

    "Modified: / 03-12-2006 / 13:30:25 / ked"
    "Created: / 21-03-2007 / 14:02:30 / janfrog"
    "Modified: / 05-12-2007 / 15:37:05 / janfrog"
    "Modified: / 18-09-2008 / 16:57:38 / Jan Vrany <vranyj1@fel.cvut.cz>"
!

test_arithmetic_multiplicative_01
    |result|

    result := interpreter basicEvaluate:'1 * 2'.
    self assert:result size = 1.
    self assert:result first type = XQuerySequenceItem typeXSInteger.
    self assert:result first value = 2.

    "Modified: / 03-12-2006 / 13:30:25 / ked"
    "Created: / 21-03-2007 / 14:03:23 / janfrog"
    "Modified: / 05-12-2007 / 15:21:57 / janfrog"
    "Modified: / 18-09-2008 / 16:57:36 / Jan Vrany <vranyj1@fel.cvut.cz>"
! !

!XQueryInterpreterTests methodsFor:'tests - constructors'!

testConstructor1
    |result|

    result := interpreter basicEvaluate:'element a { "element a" }'.
    self assert:result size = 1.
    self assert:(result first type isSubtypeOf:XQuerySequenceItem typeNode).
    self assert:result first item xpathName = 'a'.
    self assert:result first value = 'element a'.

    "Created: / 22-11-2006 / 16:50:12 / ked"
    "Modified: / 03-12-2006 / 13:43:14 / ked"
    "Modified: / 05-12-2007 / 15:27:41 / janfrog"
    "Modified: / 18-09-2008 / 16:57:37 / Jan Vrany <vranyj1@fel.cvut.cz>"
!

testConstructor3
    |result|

    result := interpreter 
                basicEvaluate:'document {
        for $i in doc("' , self class testedURI 
                        , '")/bookstore/book
        return element reduced-book {
              $i/title,
              $i/price,
              element reduced-price { $i/price * 0.9 }
            }
     }'.

    "TODO TODO TODO"

    "Created: / 22-11-2006 / 17:03:32 / ked"
    "Modified: / 02-12-2006 / 12:12:03 / ked"
    "Modified: / 24-12-2006 / 13:56:13 / janfrog"
    "Modified: / 18-09-2008 / 16:57:37 / Jan Vrany <vranyj1@fel.cvut.cz>"
!

testConstructor4
    |result|

    result := interpreter 
                basicEvaluate:'let $i := element a { element b { element c {} } }
     return $i//c/ancestor::*'.

    "TODO TODO TODO"

    "Created: / 22-11-2006 / 17:05:24 / ked"
    "Modified: / 24-12-2006 / 13:56:17 / janfrog"
    "Modified: / 18-09-2008 / 16:57:38 / Jan Vrany <vranyj1@fel.cvut.cz>"
!

testConstructor5
    |result|

    result := interpreter 
                basicEvaluate:'for $i in doc("' , self class testedURI 
                        , '")/bookstore/book
     return element { $i/title } { $i/@*, $i/* }'.

    "TODO TODO TODO"

    "Created: / 22-11-2006 / 17:07:50 / ked"
    "Modified: / 02-12-2006 / 12:12:20 / ked"
    "Modified: / 24-12-2006 / 13:56:23 / janfrog"
    "Modified: / 18-09-2008 / 16:57:36 / Jan Vrany <vranyj1@fel.cvut.cz>"
!

testConstructor6
    |result|

    result := interpreter 
                basicEvaluate:'<vycet>{ for $i in (1 to 20) where $i mod 3 eq 0 return <cislo>{ $i }</cislo> }</vycet>'.

    "TODO TODO TODO"

    "Created: / 22-11-2006 / 17:11:20 / ked"
    "Modified: / 24-12-2006 / 13:56:27 / janfrog"
    "Modified: / 18-09-2008 / 16:57:37 / Jan Vrany <vranyj1@fel.cvut.cz>"
!

testConstructor7
    |result|

   "It is important to doc function return more than one result item
        eg: <root>{doc('xmldb:attr.xml')/root/a}</root>
        which return two <a> elements"

    "TODO: how to load specified file???"
"/    result := interpreter
"/                basicEvaluate:'<root>doc("' , self class testedAttrURI 
"/                        , '")</root>/a'.
"/   self assert: (result size = 2).

    "Created: / 18-04-2009 / 20:54:48 / Jan Kurs <kursj1@fel.cvut.cz>"
    "Modified: / 19-04-2009 / 10:16:30 / Jan Kurs <kursj1@fel.cvut.cz>"
!

test_constructor_07
    |result|

    result := interpreter basicEvaluate:'element a { attribute b { "bee" } }'.
    self assert:result size = 1.
    self assert:(result first type isSubtypeOf:XQuerySequenceItem typeNode).
    self assert:result first item xpathName = 'a'.
    self assert:result first item xpathAttribute size = 1.
    self assert:result first item xpathAttribute first xpathName = 'b'.
    self assert:result first item xpathAttribute first xpathValue = 'bee'.

    "Modified: / 03-12-2006 / 13:43:14 / ked"
    "Created: / 21-03-2007 / 15:59:55 / janfrog"
    "Modified: / 05-12-2007 / 15:47:38 / janfrog"
    "Modified: / 18-09-2008 / 16:57:38 / Jan Vrany <vranyj1@fel.cvut.cz>"
!

test_direct_constructor_01
    |result|

    result := interpreter basicEvaluate:'<a/>'.
    self assert:result size = 1.
    self assert:(result first type isSubtypeOf:XQuerySequenceItem typeNode).
    self assert:result first item xpathName = 'a'.

    "Modified: / 03-12-2006 / 13:43:14 / ked"
    "Created: / 21-03-2007 / 16:16:27 / janfrog"
    "Modified: / 05-12-2007 / 15:47:38 / janfrog"
    "Modified: / 18-09-2008 / 16:57:35 / Jan Vrany <vranyj1@fel.cvut.cz>"
! !

!XQueryInterpreterTests methodsFor:'tests - functions'!

test_function_fn_abs_01

    | result |

    result := interpreter evaluate:'
    abs(-1)
    '.

    self assert:
        (result first "first item in result as document fragment"
            childNodes first "the only one child of document fragment"
                nodeValue)
                    = '1'

    "Created: / 26-12-2006 / 09:12:14 / janfrog"
    "Modified: / 18-09-2008 / 16:45:21 / Jan Vrany <vranyj1@fel.cvut.cz>"
!

test_function_fn_abs_02

    | result |

    result := interpreter evaluate:'
    abs(0)
    '.

    self assert:
        (result first "first item in result as document fragment"
            childNodes first "the only one child of document fragment"
                nodeValue)
                    = '0'

    "Created: / 26-12-2006 / 09:12:39 / janfrog"
    "Modified: / 18-09-2008 / 16:45:19 / Jan Vrany <vranyj1@fel.cvut.cz>"
!

test_function_fn_abs_03

    | result |

    result := interpreter evaluate:'
    abs(0.1)
    '.

    self assert:
        (result first "first item in result as document fragment"
            childNodes first "the only one child of document fragment"
                nodeValue)
                    = '0.1'

    "Created: / 26-12-2006 / 09:12:22 / janfrog"
    "Modified: / 18-09-2008 / 16:45:20 / Jan Vrany <vranyj1@fel.cvut.cz>"
!

test_function_fn_abs_04

    | result |

    result := interpreter evaluate:'
    fn:abs(-1)
    '.

    self assert:
        (result first "first item in result as document fragment"
            childNodes first "the only one child of document fragment"
                nodeValue)
                    = '1'

    "Created: / 28-12-2006 / 12:26:19 / janfrog"
    "Modified: / 18-09-2008 / 16:45:19 / Jan Vrany <vranyj1@fel.cvut.cz>"
!

test_function_fn_abs_05

    | result |

    result := interpreter evaluate:'
    declare namespace fnx = "http://www.w3.org/2005/xpath-functions";
    fnx:abs(-1)
    '.

    self assert:
        (result first "first item in result as document fragment"
            childNodes first "the only one child of document fragment"
                nodeValue)
                    = '1'

    "Created: / 28-12-2006 / 12:27:01 / janfrog"
    "Modified: / 18-09-2008 / 16:45:18 / Jan Vrany <vranyj1@fel.cvut.cz>"
!

test_function_fn_count_01

    | result |

    result := interpreter evaluate:'
    let $i := <a><b/><b/></a>
    return count($i//b)
    '.

    self assert:
        (result first "first item in result as document fragment"
            childNodes first "the only one child of document fragment"
                nodeValue)
                    = '2'

    "Created: / 26-12-2006 / 09:10:06 / janfrog"
    "Modified: / 18-09-2008 / 16:45:21 / Jan Vrany <vranyj1@fel.cvut.cz>"
!

test_function_user_defined_function_01

    | result |

    result := interpreter evaluate:'
    declare function f($a, $b) {
        $a + $b
    };
    f(1,1)
    '.

    self assert:
        (result first "first item in result as document fragment"
            childNodes first "the only one child of document fragment"
                nodeValue)
                    = '2'

    "Created: / 28-12-2006 / 14:01:21 / janfrog"
    "Modified: / 18-09-2008 / 16:45:19 / Jan Vrany <vranyj1@fel.cvut.cz>"
!

test_function_user_defined_function_02

    | result |

    result := interpreter evaluate:'
    declare function f($a, $b) {
        $a + $b
    };
    fn:f(1,1)
    '.

    self assert:
        (result first "first item in result as document fragment"
            childNodes first "the only one child of document fragment"
                nodeValue)
                    = '2'

    "Created: / 28-12-2006 / 14:01:44 / janfrog"
    "Modified: / 29-12-2006 / 15:56:52 / janfrog"
    "Modified: / 18-09-2008 / 16:45:21 / Jan Vrany <vranyj1@fel.cvut.cz>"
!

test_function_user_defined_function_03

    | result |

    result := interpreter evaluate:'
    declare function local:f($a) {
        $a/title
    };
    local:f(<book><title>Lord</title><author>Gandalf</author></book>)
    '.

    self assert:
        (result first "first item in result as document fragment"
            childNodes first "the only one child of document fragment"
                childNodes first "first child of result element"
                    nodeValue)
                    = 'Lord'

    "Created: / 03-01-2007 / 09:43:21 / janfrog"
    "Modified: / 21-03-2007 / 16:14:01 / janfrog"
    "Modified: / 18-09-2008 / 16:45:18 / Jan Vrany <vranyj1@fel.cvut.cz>"
!

test_function_user_defined_function_04

    | result |

    result := interpreter evaluate:'
    declare function f($a) {
        $a + 1
    };
    for $a in (1) return 
        f(10) , $a
    '.

    self assert:
        (result first "first item in result as document fragment"
            childNodes first "the only one child of document fragment"
                nodeValue)
                    = '11'

    "Modified: / 29-12-2006 / 15:56:52 / janfrog"
    "Created: / 17-09-2008 / 12:44:11 / Jan Vrany <vranyj1@fel.cvut.cz>"
    "Modified: / 18-09-2008 / 16:45:21 / Jan Vrany <vranyj1@fel.cvut.cz>"
! !

!XQueryInterpreterTests methodsFor:'tests - namespaces'!

test_namespace_01

    | result |

    result := interpreter evaluate:'
    declare namespace testns = "urn:ns:ns1";
    element testns:a { }
    '.

    self assert:
        (result first "first item in result as document fragment"
            childNodes first "the only one child of document fragment"
                namespaceURI)
                    = 'urn:ns:ns1'

    "Created: / 24-12-2006 / 15:27:57 / janfrog"
    "Modified: / 18-09-2008 / 16:45:21 / Jan Vrany <vranyj1@fel.cvut.cz>"
!

test_namespace_02

    | result |

    result := interpreter evaluate:'
    declare default element namespace "urn:ns:ns1";
    element a { }
    '.

    self assert:
        (result first "first item in result as document fragment"
            childNodes first "the only one child of document fragment"
                namespaceURI)
                    = 'urn:ns:ns1'

    "Created: / 24-12-2006 / 17:20:15 / janfrog"
    "Modified: / 18-09-2008 / 16:45:18 / Jan Vrany <vranyj1@fel.cvut.cz>"
!

test_namespace_03

    | result |

    result := interpreter evaluate:'
    declare namespace testns = "urn:ns:ns1";
    <testns:a />
    '.

    self assert:
        (result first "first item in result as document fragment"
            childNodes first "the only one child of document fragment"
                namespaceURI)
                    = 'urn:ns:ns1'

    "Created: / 24-12-2006 / 17:26:53 / janfrog"
    "Modified: / 18-09-2008 / 16:45:17 / Jan Vrany <vranyj1@fel.cvut.cz>"
!

test_namespace_04

    | result |

    result := interpreter evaluate:'
    <a/>
    '.

    self assert:
        (result first "first item in result as document fragment"
            childNodes first "the only one child of document fragment"
                namespaceURI)
                    = ''

    "Created: / 24-12-2006 / 17:29:02 / janfrog"
    "Modified: / 18-09-2008 / 16:45:19 / Jan Vrany <vranyj1@fel.cvut.cz>"
!

test_namespace_05

    | result |

    result := interpreter evaluate:'
    declare namespace testns = "urn:ns:testns";            
    let $a := <root><testns:a/></root>
    return $a/testns:a
    '.

    self assert:
        (result first "first item in result as document fragment"
            childNodes first "the only one child of document fragment"
                namespaceURI)
                    = 'urn:ns:testns'

    "Created: / 24-12-2006 / 17:30:18 / janfrog"
    "Modified: / 18-09-2008 / 16:45:19 / Jan Vrany <vranyj1@fel.cvut.cz>"
!

test_namespace_06

    | result |

    result := interpreter evaluate:'
    declare namespace testns = "urn:ns:nsA";            
    doc("file:///namespaces.xml")//testns:a
    '.

    self assert:
        (result first "first item in result as document fragment"
            childNodes first "the only one child of document fragment"
                namespaceURI)
                    = 'urn:ns:nsA'

    "Created: / 24-12-2006 / 17:42:34 / janfrog"
    "Modified: / 18-09-2008 / 16:45:19 / Jan Vrany <vranyj1@fel.cvut.cz>"
!

test_namespace_07

    | result |

    result := interpreter evaluate:'
    declare namespace testns = "urn:ns:nsB";            
    doc("file:///namespaces.xml")//testns:b
    '.

    self assert:
        (result first "first item in result as document fragment"
            childNodes first "the only one child of document fragment"
                namespaceURI)
                    = 'urn:ns:nsB'

    "Created: / 24-12-2006 / 17:42:53 / janfrog"
    "Modified: / 18-09-2008 / 16:45:20 / Jan Vrany <vranyj1@fel.cvut.cz>"
!

test_namespace_08

    | result |

    result := interpreter evaluate:'
    declare namespace testns = "urn:ns:nsD";            
    doc("file:///namespaces.xml")//c[@testns:d]
    '.

    self assert:
        (result first "first item in result as document fragment"
            childNodes first "the only one child of document fragment"
                localName)
                    = 'c'

    "Created: / 24-12-2006 / 17:59:07 / janfrog"
    "Modified: / 18-09-2008 / 16:45:20 / Jan Vrany <vranyj1@fel.cvut.cz>"
! !

!XQueryInterpreterTests methodsFor:'tests - xpath'!

testPathExpr1
    |result|

    result := interpreter 
                basicEvaluate:'doc("file:///bookstore.xml")/bookstore/book/title'.
    self assert:result size = 3.
    self assert:(result first type isSubtypeOf:XQuerySequenceItem typeNode).
    self assert:result first value = 'Harry Potter'.
    self assert:(result second type isSubtypeOf:XQuerySequenceItem typeNode).
    self assert:result second value = 'Learning XML'.
    self assert:(result third type isSubtypeOf:XQuerySequenceItem typeNode).
    self assert:result third value = 1984.

    "/   #('Harry Potter' 'Learning XML' 1984)

    "Created: / 08-10-2006 / 21:23:52 / ked"
    "Modified: / 03-12-2006 / 13:40:09 / ked"
    "Modified: / 05-12-2007 / 15:47:38 / janfrog"
    "Modified: / 18-09-2008 / 16:57:38 / Jan Vrany <vranyj1@fel.cvut.cz>"
!

testPathExpr2
    |result|

    result := interpreter basicEvaluate:'doc("file:///bookstore.xml")//price'.
    self assert:result size = 3.
    self assert:(result first type isSubtypeOf:XQuerySequenceItem typeNode).
    self assert:result first value = 29.99.
    self assert:(result second type isSubtypeOf:XQuerySequenceItem typeNode).
    self assert:result second value = 39.95.
    self assert:(result third type isSubtypeOf:XQuerySequenceItem typeNode).
    self assert:result third value = 19.84.

    "Created: / 08-10-2006 / 21:24:21 / ked"
    "Modified: / 03-12-2006 / 13:40:34 / ked"
    "Modified: / 05-12-2007 / 15:47:38 / janfrog"
    "Modified: / 18-09-2008 / 16:57:36 / Jan Vrany <vranyj1@fel.cvut.cz>"
!

testPathExpr3
    |result aux|

    result := interpreter 
                basicEvaluate:'for $book in doc("file:///bookstore.xml")/bookstore/book
     where $book/price > 30
     return ($book/title, $book/price, $book/price * 0.9)'.
    self assert:result size = 3.
    self assert:(result first type isSubtypeOf:XQuerySequenceItem typeNode).
    self assert:result first value = 'Learning XML'.
    self assert:(result second type isSubtypeOf:XQuerySequenceItem typeNode).
    self assert:result second value = 39.95.
    self 
        assert:(result third type isSubtypeOf:XQuerySequenceItem typeXSFloat).
    aux := result third value - 35.955.
    self assert:aux < 0.001.

    "Created: / 08-10-2006 / 21:22:44 / ked"
    "Modified: / 03-12-2006 / 13:43:44 / ked"
    "Modified: / 05-12-2007 / 15:56:09 / janfrog"
    "Modified: / 18-09-2008 / 16:57:37 / Jan Vrany <vranyj1@fel.cvut.cz>"
!

testPathExpr4
    |result|

    result := interpreter 
                basicEvaluate:'doc("file:///bookstore.xml")/bookstore/book[2]/preceding-sibling::*/title/text()'.
    self assert:result size = 1.
    self assert:(result first type isSubtypeOf:XQuerySequenceItem typeNode).
    self assert:result first item xpathIsText.
    self assert:result first item xpathValue = 'Harry Potter'.

    "Created: / 22-11-2006 / 16:14:22 / ked"
    "Modified: / 03-12-2006 / 13:41:16 / ked"
    "Modified: / 05-12-2007 / 15:47:38 / janfrog"
    "Modified: / 18-09-2008 / 16:57:37 / Jan Vrany <vranyj1@fel.cvut.cz>"
!

testPathExpr5
    |result|

    result := interpreter 
                basicEvaluate:'doc("file:///bookstore.xml")/bookstore/book[1]/title/@*'.
    self assert:result size = 2.
    self assert:(result 
                anySatisfy:[:dataContextItem | 
                    (((dataContextItem type isSubtypeOf:XQuerySequenceItem typeNode) 
                        & dataContextItem item xpathIsAttribute & dataContextItem item xpathName) 
                        = 'lang' & dataContextItem item xpathValue) 
                        = 'eng'
                ]).
    self assert:(result 
                anySatisfy:[:dataContextItem | 
                    (((dataContextItem type isSubtypeOf:XQuerySequenceItem typeNode) 
                        & dataContextItem item xpathIsAttribute & dataContextItem item xpathName) 
                        = 'withPictures' & dataContextItem item xpathValue) 
                        = 'yes'
                ]).

    "Created: / 22-11-2006 / 16:17:58 / ked"
    "Modified: / 03-12-2006 / 13:41:33 / ked"
    "Modified: / 05-12-2007 / 15:47:38 / janfrog"
    "Modified: / 18-09-2008 / 16:57:38 / Jan Vrany <vranyj1@fel.cvut.cz>"
!

testPredicate1
    |result|

    result := interpreter basicEvaluate:'(1 to 100)[. mod 5 eq 0][. < 11]'.
    self assert:result size = 2.
    self 
        assert:(result first type isSubtypeOf:XQuerySequenceItem typeXSInteger).
    self assert:result first value = 5.
    self 
        assert:(result second type isSubtypeOf:XQuerySequenceItem typeXSInteger).
    self assert:result second value = 10.

    "Created: / 22-11-2006 / 16:39:11 / ked"
    "Modified: / 03-12-2006 / 13:41:50 / ked"
    "Modified: / 05-12-2007 / 15:37:06 / janfrog"
    "Modified: / 18-09-2008 / 16:57:36 / Jan Vrany <vranyj1@fel.cvut.cz>"
!

testPredicate2
    |result|

    result := interpreter 
                basicEvaluate:'doc("file:///bookstore.xml")/bookstore/book[1]/title'.
    self assert:result size = 1.
    self assert:(result first type isSubtypeOf:XQuerySequenceItem typeNode).
    self assert:result first value = 'Harry Potter'.

    "Created: / 22-11-2006 / 16:41:23 / ked"
    "Modified: / 03-12-2006 / 13:42:02 / ked"
    "Modified: / 05-12-2007 / 15:47:38 / janfrog"
    "Modified: / 18-09-2008 / 16:57:37 / Jan Vrany <vranyj1@fel.cvut.cz>"
!

testPredicate3
    |result|

    result := interpreter 
                basicEvaluate:'doc("file:///bookstore.xml")/bookstore/book[price > 30]/title'.
    self assert:result size = 1.
    self assert:(result first type isSubtypeOf:XQuerySequenceItem typeNode).
    self assert:result first value = 'Learning XML'.

    "Created: / 22-11-2006 / 16:42:01 / ked"
    "Modified: / 03-12-2006 / 13:42:13 / ked"
    "Modified: / 05-12-2007 / 15:47:38 / janfrog"
    "Modified: / 18-09-2008 / 16:57:37 / Jan Vrany <vranyj1@fel.cvut.cz>"
!

test_index_01_low
    |result|

    result := interpreter 
                basicEvaluate:'(element a { element b { element c { "c" } } })/*/* '.
    self assert:(result size = 1).
    self assert:(result first item nodeName localName = 'c')

    "Modified: / 18-09-2008 / 16:57:35 / Jan Vrany <vranyj1@fel.cvut.cz>"
!

test_xpath_01
    |result|

    result := interpreter 
                basicEvaluate:'doc("file:///bookstore.xml")/bookstore'.
    self assert:result size = 1.

    "Modified: / 03-12-2006 / 13:40:09 / ked"
    "Created: / 21-03-2007 / 18:45:28 / janfrog"
    "Modified: / 18-11-2007 / 09:10:18 / janfrog"
    "Modified: / 18-09-2008 / 16:57:36 / Jan Vrany <vranyj1@fel.cvut.cz>"
! !

!XQueryInterpreterTests class methodsFor:'documentation'!

version
    ^ '$Header: /opt/data/cvs/stx/goodies/xmlsuite/xquery/XQuery__XQueryExecutorTests.st,v 1.8 2007-12-05 21:07:45 vranyj1 Exp $'
! !
